#!/usr/bin/env python
# Copyright (C) 2010-2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
from iti1480a.parser import *
import signal
import sys
import errno
import select
import fcntl
import os

COLOR_GREEN = '\x1b[32m'
COLOR_STRONG_GREEN = '\x1b[1;32m'
COLOR_YELLOW = '\x1b[1;33m'
COLOR_RED = '\x1b[31m'
COLOR_STRONG_RED = '\x1b[1;31m'
COLOR_BLUE = '\x1b[1;34m'
COLOR_ORANGE = '\x1b[33m'

TOKEN_COLOR = {
    TOKEN_TYPE_OUT: COLOR_BLUE,
    TOKEN_TYPE_IN: COLOR_BLUE,
    TOKEN_TYPE_SETUP: COLOR_BLUE,
    TOKEN_TYPE_SSPLIT: COLOR_BLUE,
    TOKEN_TYPE_CSPLIT: COLOR_BLUE,

    TOKEN_TYPE_ACK: COLOR_STRONG_GREEN,
    TOKEN_TYPE_NAK: COLOR_RED,
    TOKEN_TYPE_STALL: COLOR_STRONG_RED,

    TOKEN_TYPE_NYET: COLOR_YELLOW,
    TOKEN_TYPE_PING: COLOR_YELLOW,

    TOKEN_TYPE_DATA0: COLOR_GREEN,
    TOKEN_TYPE_DATA1: COLOR_GREEN,
    TOKEN_TYPE_DATA2: COLOR_GREEN,
    TOKEN_TYPE_MDATA: COLOR_GREEN,

    TOKEN_TYPE_SOF: COLOR_ORANGE,
}

def hexdump(data):
    result = []
    append = result.append
    offset = 0
    while data:
        line = data[:16]
        half1 = line[:8]
        half2 = line[8:]
        data = data[16:]
        append(
            ' ' * 20
            + ('\x1b[33m%03x \x1b[0;36m' % offset)
            + ' '.join([
                '%02x' % ord(x) for x in half1
            ] + ['  '] * (8 - len(half1)))
            + '  '
            + ' '.join([
                '%02x' % ord(x) for x in half2
            ] + ['  '] * (8 - len(half2)))
            + ' '
            + ''.join(x if x.isalnum() or x == ' ' else '.' for x in line)
            + ' ' * (16 - len(line))
            + '\x1b[0m'
        )
        offset += 16
    return '\n'.join(result)

class HumanReadable(object):
    def __init__(self, write, error, verbosity):
        self._write = write
        self._error_write = error
        self._verbosity = verbosity
        self._sof_start = (None, None)
        self._sof_count = 0
        self._sof_major = None
        self._sof_minor = None
        self._last_tic = None
        noop = lambda _, __: None
        self._dispatch = {
            MESSAGE_RAW: (lambda _, x: x) if verbosity > -1 else noop,
            MESSAGE_RESET: self._reset if verbosity > -1 else noop,
            MESSAGE_TRANSACTION: self._transaction,
            MESSAGE_INCOMPLETE: (
                lambda x, y: self._transaction(x, y, incomplete=True)
            ) if verbosity > 0 else noop,
            MESSAGE_TRANSACTION_ERROR: self._error,
            MESSAGE_LS_EOP: self._ls_eop if verbosity > 2 else noop,
            MESSAGE_FS_EOP: self._fs_eop if verbosity > 2 else noop,
            MESSAGE_FS_TO_CHIRP: self._fs_to_chirp,
        }

    def _print(self, tic, printable, write):
        if tic is None:
            time = '?'
        else:
            time = tic_to_time(tic)
            if tic < self._last_tic:
                time = '\x1b[31m' + time + '\x1b[0m'
            else:
                self._last_tic = tic
        write(time + ' ' + printable + '\n')

    def _printSOFCount(self):
        sof_tic, sof_start_frame = self._sof_start
        self._print(
            sof_tic,
            TOKEN_COLOR[TOKEN_TYPE_SOF] + 'SOF (%i) %s -> %i.%i\x1b[0m' % (
                self._sof_count,
                sof_start_frame,
                self._sof_major,
                self._sof_minor,
            ), self._write)
        self._sof_count = 0

    def push(self, tic, message_type, data):
        printable = self._dispatch[message_type](tic, data)
        if printable is not None:
            if self._sof_count:
                self._printSOFCount()
            self._print(tic, printable, self._write)

    def _error(self, tic, data):
        self._print(tic, '\x1b[41m%r\x1b[0m' % (data, ), self._error_write)

    @staticmethod
    def _reset(_, data):
        return '\x1b[35mDevice reset (%s)\x1b[0m' % (short_tic_to_time(data), )

    @staticmethod
    def _fs_to_chirp(_, data):
        return '\x1b[35mFS to chirp trigger (%s)\x1b[0m' % (
            short_tic_to_time(data),
        )

    @staticmethod
    def _ls_eop(_, data):
        return '\x1b[33mLS EOP (%s)\x1b[0m' % (short_tic_to_time(data), )

    @staticmethod
    def _fs_eop(_, data):
        return '\x1b[33mFS EOP (%s)\x1b[0m' % (short_tic_to_time(data), )

    def _transaction(self, tic, data, incomplete=False):
        if data[0][0] == TOKEN_TYPE_SOF:
            sof_data, = data
            try:
                decoded = decode(sof_data)
            except IndexError:
                return
            if not decoded['crc_error']:
                if self._verbosity < 2:
                    return
                frame = decoded['frame']
                if frame == self._sof_major:
                    self._sof_minor += 1
                else:
                    self._sof_major = frame
                    self._sof_minor = 0
                if self._verbosity <= 3:
                    if not self._sof_count:
                        self._sof_start = (
                            tic,
                            '%i.%i' % (frame, self._sof_minor),
                        )
                    self._sof_count += 1
                    return
        if self._verbosity < 1 and (
                    data[-1][0] == TOKEN_TYPE_NAK or (
                        data[0][0] == TOKEN_TYPE_SSPLIT and
                        len(data) == 2
                    ) or (
                        data[0][0] == TOKEN_TYPE_CSPLIT and
                        data[-1][0] == TOKEN_TYPE_NYET
                    )
                ):
            return
        result = ''
        packet_data = None
        for packet in data:
            try:
                decoded = decode(packet)
            except IndexError:
                break
            try:
                result += TOKEN_COLOR[packet[0]]
            except KeyError:
                assert packet[0] == TOKEN_TYPE_PRE_ERR
                # ERR if part of a SPLIT transaction, PRE otherwise.
                # Color & name appropriately.
                result += COLOR_RED + 'ERR' \
                    if data[0][0] in (TOKEN_TYPE_SSPLIT, TOKEN_TYPE_CSPLIT) \
                    else COLOR_GREEN + 'PRE'
            else:
                result += decoded['name'].ljust(7)
            result += '\x1b[0m '
            if 'endpoint' in decoded:
                result += '@%03i.%02i ' % (
                    decoded['address'],
                    decoded['endpoint'],
                )
            elif 'port' in decoded:
                result += '@%03i:%03i %-11s ' % (
                    decoded['address'],
                    decoded['port'],
                    decoded['endpoint_type'],
                )
                if 'continuation' in decoded:
                    result += '%-9s ' % decoded['continuation']
                else:
                    result += 'LS ' if decoded['speed'] else 'HS '
            elif 'data' in decoded:
                if self._verbosity >= 0:
                    assert packet_data is None
                    packet_data = decoded['data']
                result += '%3iB ' % len(decoded['data'])
            elif 'frame' in decoded:
                frame = decoded['frame']
                result += '%4i%s ' % (
                    frame,
                    ('' if decoded['crc_error'] else '.%i' % self._sof_minor),
                )
            if decoded.get('crc_error'):
                result += '\x1b[1;31mCRC error\x1b[0m '
        if incomplete:
            result += '\x1b[1;31m(incomplete transaction)\x1b[0m'
        if packet_data:
            result += '\n' + hexdump(packet_data)
        return result

    def stop(self):
        if self._sof_count:
            self._printSOFCount()

CHUNK_SIZE = 16 * 1024
def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option(
        '-v', '--verbose', action='count',
        default=0, help='Increase verbosity',
    )
    parser.add_option(
        '-q', '--quiet', action='count',
        default=0, help='Decrease verbosity',
    )
    parser.add_option(
        '-i', '--infile', default='-',
        help='Data source (default: stdin)',
    )
    parser.add_option(
        '-o', '--outfile', default='-',
        help='Data destination (default: stdout)',
    )
    parser.add_option(
        '-t', '--tee', help='Also write raw input to that '
        'file. Useful as tee(1) doesn\'t close its stdin when its stdout '
        'gets closed, so next process (ie, this program) does not know it '
        'should exit.',
    )
    parser.add_option('-f', '--follow', action='store_true',
        help='Ignore SIGINT & SIGTERM so all input is read.')
    (options, args) = parser.parse_args()
    if options.infile == '-':
        infile = sys.stdin
    else:
        try:
            infile = open(options.infile, 'r')
        except IOError:
            print >>sys.stderr, 'Could not open --infile %r' % (
                options.infile,
            )
            sys.exit(1)
    if options.outfile == '-':
        write = sys.stdout.write
    else:
        try:
            write = open(options.outfile, 'w').write
        except IOError:
            print >>sys.stderr, 'Could not open --outfile %r' % (
                options.outfile,
            )
            sys.exit(1)
    if options.tee:
        try:
            raw_write = open(options.tee, 'w').write
        except IOError:
            print >>sys.stderr, 'Could not open --tee %r' % (options.tee, )
            sys.exit(1)
    else:
        raw_write = lambda x: None
    human_readable = HumanReadable(write, sys.stderr.write,
        options.verbose - options.quiet)
    stream = ReorderedStream(
        Packetiser(
            TransactionAggregator(
                human_readable,
                human_readable.push,
            ),
            human_readable.push
        )
    )
    push = stream.push
    if options.follow:
        for sig in (signal.SIGINT, signal.SIGTERM):
            signal.signal(sig, signal.SIG_IGN)
    fcntl.fcntl(
        infile,
        fcntl.F_SETFL,
        fcntl.fcntl(infile, fcntl.F_GETFL) | os.O_NONBLOCK,
    )
    rlist = [infile]
    wlist = elist = []
    read = infile.read
    try:
        while True:
            try:
                data = read(CHUNK_SIZE)
            except IOError, exc:
                if exc.errno != errno.EAGAIN:
                    raise
                # Using select instead of more recent alternatives, because:
                # - we wait on one file descriptor, which is likely to have a
                #   very low value (1 or 4), so bad performance is not really
                #   an issue.
                # - although this is ITI1480A-*linux*, I do not want to
                #   alienate BSD users by relying on epoll.
                # Ignore return value, error is detected by empty read.
                select.select(rlist, wlist, elist)
                continue
            if not data:
                break
            raw_write(data)
            try:
                push(data)
            except ParsingDone:
                break
        stream.stop()
    except IOError, exc:
        # Happens when output is piped to a pager, and pager exits before stdin
        # is fully parsed.
        if exc.errno != errno.EPIPE:
            raise
    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    main()




# Copyright (C) 2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
from ply.yacc import *

def startPush(self):
    """
    Incremental parsing: Reinitialise parser state.
    """
    self._push_pslice = pslice = YaccProduction(None)
    self._push_error_count = 0
    pslice.parser = self
    self.statestack = [0]
    end = YaccSymbol()
    end.type = '$end'
    self.symstack = [end]
LRParser.startPush = startPush

def push(self, token):
    """
    Incremental parsing: Update parser state with given token.
    """
    if token is None:
        token = YaccSymbol()
        token.type = '$end'
    symstack = self.symstack
    statestack = self.statestack
    actions = self.action
    token_type = token.type
    productions = self.productions
    pslice = self._push_pslice
    goto = self.goto
    while True:
        state_actions = actions[statestack[-1]]
        try:
            t = state_actions[token_type]
        except KeyError:
            errorcount = self._push_error_count
            if errorcount == 0 or self.errorok:
                self.errorok = 0
                errtoken = token
                if token_type == '$end':
                    errtoken = None
                if self.errorfunc:
                    tok = self.errorfunc(errtoken)
                    if self.errorok:
                        # For compatibility with ply.yacc API.
                        # errorfunc could call push instead.
                        self.push(tok)
                        return
                else:
                    if errtoken:
                        sys.stderr.write("yacc: Syntax error at line %d, token=%s\n" % (getattr(token, 'lineno', 0), token_type))
                    else:
                        sys.stderr.write("yacc: Parse error in input. EOF\n")
                        return
            else:
                self._push_error_count = error_count
            if token_type == "$end":
                return
            elif len(statestack) <= 1:
                statestack[:] = [0]
                return
            elif token_type == 'error':
                symstack.pop()
                statestack.pop()
            else:
                sym = symstack[-1]
                if sym.type == 'error':
                    return
                t = YaccSymbol()
                t.type = 'error'
                if hasattr(token, "lineno"):
                    t.lineno = token.lineno
                t.value = token
                self.push(t)
            continue
        if t > 0:
            # shift a symbol on the stack
            statestack.append(t)
            symstack.append(token)
            if self._push_error_count:
                self._push_error_count -= 1
        elif t < 0:
            # reduce a symbol on the stack
            p = productions[-t]
            pname = p.name
            plen = p.len
            sym = YaccSymbol()
            sym.name = pname
            sym.value = None
            sym.type = None
            if plen:
                targ = symstack[-plen-1:]
                targ[0] = sym
                del symstack[-plen:]
                del statestack[-plen:]
            else:
                targ = [sym]
            pslice.slice = targ
            p.callable(pslice)
            statestack.append(goto[statestack[-1]][pname])
            symstack.append(sym)
            continue
        break
LRParser.push = push



#!/usr/bin/env python
# Copyright (C) 2010-2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
from struct import unpack, calcsize
from cStringIO import StringIO

MAGIC = "CSPT"
UNKNOWN_1 = '\x20\x00\x00\x00\x00\x00\x00\x50'
UNKNOWN_2 = '\xa0\x70'
UNKNOWN_3 = '\x00\x00\x6c\x65'
UNKNOWN_4 = '\x0f\x00\x00\x00'
HEADER_FORMAT = '<4sI8s2sH4s4sI'
HEADER_LEN = calcsize(HEADER_FORMAT)

CPUCS_OFFSET = 0xe600
CPUCS_RUN = '\x00'
CPUCS_PAUSE = '\x01'

def iterSPTCommands(spt_file, stop_on_cpu_start=False):
    """
    Iterate over USB commands stored in SPT file format.

    spt_file (file-ish object)
        File object to iterate in.
        Should be positionned at the beginning of a command.
        Read-only access is sufficient.
    stop_on_cpu_start (bool)
        If False (default), dump all commands in a single generator run.
        If True, accesses to CPUCS register with a 0x00 value will cause the
        generator to stop without yielding that command. Upon such stop, file
        position will be on the command following the 0x00 write to CPUCS.
        Also, this makes the generator skip writes to CPUCS with a 0x01 value
        for convenience.

        This is usefull to extract individual programs from a composite file.

    Generated values:
        2-tuple composed of section offset and section data.
    """
    read = spt_file.read
    while True:
        header = read(HEADER_LEN)
        if not header:
            # End of file
            break
        magic, packet_length, unknown_1, unknown_2, offset, unknown_3, \
            unknown_4, data_length = unpack(HEADER_FORMAT, header)
        # Check header contains expected values
        assert magic == MAGIC, repr(magic)
        assert unknown_1 == UNKNOWN_1, repr(unknown_1)
        assert unknown_2 == UNKNOWN_2, repr(unknown_2)
        assert unknown_3 == UNKNOWN_3, repr(unknown_3)
        assert unknown_4 == UNKNOWN_4, repr(unknown_4)
        assert packet_length - data_length == 32, (packet_length, data_length)
        data = read(data_length)
        assert len(data) == data_length, (len(data), data_length)
        if stop_on_cpu_start and offset == CPUCS_OFFSET:
            if data == CPUCS_RUN:
                break
            elif data == CPUCS_PAUSE:
                continue
            else:
                raise ValueError('Unknown CPUCS register value: %r' % (data, ))
        yield (offset, data)

IHX_MAX_LINE_LENGTH = 0x10

def toIntelHex(spt_file):
    """
    spt_file (file)
        The file to read, in SPT format.

    Return value:
        List of strings representing IHX file data.
        Separation between 2 entries in the list is disctated by the presence
        of an explicit write to 1 at CPUCS address in input file.
    """
    ihx_list = []
    while True:
        memory = {}
        for offset, data in iterSPTCommands(spt_file):
            # Handle writes to CPUCS here, so we can detect end of file.
            if offset == CPUCS_OFFSET:
                if data == CPUCS_RUN:
                    break
                elif data == CPUCS_PAUSE:
                    continue
                else:
                    raise ValueError('Unknown CPUCS register value: %r' % (
                        data, ))
            if offset in memory:
                raise ValueError('Code overwriting itself: 0x%x' % (offset, ))
            memory[offset] = data
        if memory:
            ihx_file = StringIO()
            write = ihx_file.write
            offset_list = memory.keys()
            offset_list.sort()
            next_offset = 0
            for offset in offset_list:
                if offset < next_offset:
                    raise ValueError('Code overwriting itself: 0x%x < 0x%x' % (
                        offset, next_offset))
                data = memory[offset]
                next_offset = offset
                while data:
                    chunk = data[:IHX_MAX_LINE_LENGTH]
                    data = data[IHX_MAX_LINE_LENGTH:]
                    chunk_len = len(chunk)
                    line = [
                        chunk_len,
                        (next_offset >> 8) & 0xff,
                        next_offset & 0xff, 0
                    ] + [ord(x) for x in chunk]
                    line.append((-sum(line)) & 0xff)
                    write(':')
                    write(''.join('%02X' % x for x in line))
                    write('\n')
                    next_offset += chunk_len
            write(':00000001FF') # end of file
            ihx_list.append(ihx_file.getvalue())
        else:
            break
    return ihx_list

def main():
    import sys
    from os.path import basename, splitext
    argv = sys.argv
    if len(argv) != 2:
        print 'Usage: %s <file.spt>\n  Result: <file_0.ihx> [<file_1.ihx> ' \
            '[...]]' % (argv[0], )
        sys.exit(1)

    infile_name = argv[1]
    prefix = splitext(basename(infile_name))[0]
    for index, outfile_data in enumerate(toIntelHex(open(infile_name, 'rb'))):
        open('%s_%i.ihx' % (prefix, index), 'w').write(outfile_data)

if __name__ == '__main__':
    main()




#!/usr/bin/python
from iti1480a.parser import *
import argparse
import errno
import sys

def parseTime(value):
    """
    Parse a string timestamp representation into a tic count.
    """
    if ':' in value:
        minute, second = value.split(':')
        minute = int(minute)
    else:
        minute = 0
        second = value
    if '.' in second:
        second, remainder = second.split('.')
        second = int(second)
    else:
        second = 0
        remainder = value
    for separator in '\'"n':
        remainder = remainder.replace(separator, '')
    return (
            (minute * 60 + seconds) * 1000000000 +
            int(remainder.ljust(9, '0')[:9])
        ) / TIME_INITIAL_MULTIPLIER

def parsePipe(value):
    try:
        address, endpoint, direction = value.split('.')
    except ValueError:
        raise argparse.ArgumentTypeError('Malformed pipe identifier')
    if address:
        try:
            address = int(address)
        except TypeError:
            raise argparse.ArgumentTypeError('Non-numeric device addres')
    else:
        address = None
    try:
        endpoint = int(endpoint)
    except TypeError:
        raise argparse.ArgumentTypeError('Non-numeric endpoint')
    direction = direction.lower()
    if direction not in ('i', 'o'):
        raise argparse.ArgumentTypeError('Invalid pipe direction')
    return PipeMatcher(address, endpoint, direction)

class PipeMatcher(object):
    __slots__ = ('address', 'endpoint', 'direction')
    def __init__(address, endpoint, direction):
        self.address = address
        self.endpoint = endpoint
        self.direction = direction

    def matchEndpoint(self, address, endpoint, direction):
        if endpoint != self.endpoint or direction != self.direction:
            return False
        if address and self.address == None:
            self.address = address
        return address == self.address

class DummyHub(object):
    def __init__(self, address):
        pass

    def push(self, tic, transaction_type, data):
        pass

class Pipe(object):
    __slots__ = ('out')
    def __init__(self, out):
        self.out = out

    def push(self, tic, transaction_type, data):
        pass

CHUNK_SIZE = 16 * 1024
def main():
    parser = argparse.ArgumentParser(
        description='Extract a specific data stream from a capture.',
    )
#    parser.add_argument(
#        '-l', '--list', action='store_true',
#        help='List available data streams (disable data dump).',
#    )
    parser.add_argument(
        '-f', '--from', type=parseTime, default=0,
        help='Capture timestamp to start dumping at (included). '
            'Example: "001:13.705\'962\\"900n". Sub-millisecond separators, '
            'minute leading zeros and second trailing zeros are optional, '
            'so an equivalent representation would be "1:13.705962".',
    )
    parser.add_argument(
        '-t', '--to', type=parseTime, default=float('inf'),
        help='Capture timestamp to stop dumping at (excluded).',
    )
    parser.add_argument(
        '-i', '--infile', default=sys.stdin, type=argparse.FileType('r'),
        help='Data source (default: stdin)',
    )
    parser.add_argument(
        '-o', '--outfile', default=sys.stdout, type=argparse.FileType('w'),
        help='Data destination (default: stdout)',
    )
    parser.add_argument(
        'pipe', type=parsePipe, nargs=1,
        help='Device address, endpoint and direction of trafic to dump. '
            'Examples: "14.2.o" dumps outgoing traffic (host-to-device) for '
            'endpoint 2 of device at address 14. "14.2.i" dumps incomming '
            '(device-to-host) traffic for the same endpoint and device. '
            'An empty device address matches the first non-zero address '
            'transfering data for given endpoint and direction, in considered '
            'capture time span.',
    )
    args = parser.parse_args()
    infile = options.infile
    write = options.outfile
    discard = lambda x: None
    def newPipe(address, endpoint):
        raise NotImplementedError
    stream = ReorderedStream(
        Packetiser(
            TransactionAggregator(
                PipeAggregator(
                    discard,
                    discard,
                    DummyHub,
                    newPipe,
                ),
                discard,
            ),
            discard,
        )
    )
    push = stream.push
    read = infile.read
    try:
        while True:
            data = read(CHUNK_SIZE)
            if not data:
                break
            try:
                push(data)
            except ParsingDone:
                break
        stream.stop()
    except IOError, exc:
        # Happens when output is piped to a pager, and pager exits before stdin
        # is fully parsed.
        if exc.errno != errno.EPIPE:
            raise
    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# Copyright (C) 2010-2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
import sys
import os
import usb1
from struct import pack, unpack
import time
import signal
import errno

VENDOR_ID = 0x16C0
DEVICE_ID = 0x07A9

def getDeviceHandle(context, vendor_id, device_id, usb_device=None):
    if usb_device is None:
        return context.openByVendorIDAndProductID(vendor_id, device_id)
    bus_number, device_address = usb_device
    for device in context.getDeviceList():
        if bus_number != device.getBusNumber() \
              or device_address != device.getDeviceAddress():
            continue
        if (device.getVendorID() == vendor_id and
            device.getProductID() == device_id):
            return device.open()
        raise ValueError(
            'Device at %03i.%03i is not of expected type: '
            '%04x.%04x, %04x.%04x expected' %
            usb_device + (vendor_id, device_id),
        )

class BaseUSBAnalyzer(object):
    def __init__(self, usb_handle):
        self._handle = usb_handle

    def sendFirmware(self, firmware_file):
        read = firmware_file.read
        write = self.writeCommand

        write(self.COMMAND_FPGA, self.COMMAND_FPGA_CONFIGURE_START)
        try:
            # Empty device FIFO, discarding data.
            self._handle.bulkRead(2, 2048, 10)
            self._handle.bulkRead(2, 2048, 10)
        except usb1.USBErrorTimeout:
            pass
        else:
            raise Exception('Read 2k, EP2 FIFO still not empty')
        while True:
            conf_data = read(self.COMMAND_DATA_LEN)
            if not conf_data:
                break
            write(
                self.COMMAND_FPGA,
                self.COMMAND_FPGA_CONFIGURE_WRITE,
                conf_data,
            )
        write(self.COMMAND_FPGA, self.COMMAND_FPGA_CONFIGURE_STOP)

    def stopCapture(self):
        self.writeCommand(self.COMMAND_STOP)

    def getStatus(self):
        return ord(self.readCommand(1, self.COMMAND_STATUS))

    def pauseCapture(self):
        self.writeCommand(self.COMMAND_PAUSE, self.COMMAND_PAUSE_PAUSE)

    def continueCapture(self):
        self.writeCommand(self.COMMAND_PAUSE, self.COMMAND_PAUSE_CONTINUE)

class CompatibleUSBAnalyzer(BaseUSBAnalyzer):
    COMMAND_DATA_LEN = 61
    COMMAND_PAUSE = '\x03'
    COMMAND_PAUSE_CONTINUE = '\x00'
    COMMAND_PAUSE_PAUSE = '\x01'
    COMMAND_STATUS = '\x02'
    COMMAND_STOP = '\x01'
    COMMAND_FPGA = '\x00'
    COMMAND_FPGA_CONFIGURE_START = '\x00'
    COMMAND_FPGA_CONFIGURE_WRITE = '\x01'
    COMMAND_FPGA_CONFIGURE_STOP = '\x02'

    def writeCommand(self, command, sub_command='\x00', data=''):
        data_len = len(data)
        if data_len < self.COMMAND_DATA_LEN:
            data = data + '\x00' * (self.COMMAND_DATA_LEN - data_len)
        to_write = ''.join((command, sub_command, data, pack('B', data_len)))
        assert len(to_write) == 64, repr(to_write)
        self._handle.bulkWrite(1, to_write)

    def readCommand(self, length, command, sub_command='\x00'):
        self.writeCommand(command, sub_command)
        return self._handle.bulkRead(1, 64)[:length]

class CompliantUSBAnalyzer(BaseUSBAnalyzer):
    COMMAND_DATA_LEN = 0x1000 # XXX: Why error on larger values ?
    VENDOR_COMMAND = 0x10
    COMMAND_PAUSE = 3
    COMMAND_PAUSE_CONTINUE = 0
    COMMAND_PAUSE_PAUSE = 1
    COMMAND_STATUS = 2
    COMMAND_STOP = 1
    COMMAND_FPGA = 0
    COMMAND_FPGA_CONFIGURE_START = 0
    COMMAND_FPGA_CONFIGURE_WRITE = 1
    COMMAND_FPGA_CONFIGURE_STOP = 2
    COMMAND_MEMORY = 0xff
    COMMAND_MEMORY_INTERNAL = 0
    COMMAND_MEMORY_EXTERNAL = 1
    COMMAND_MEMORY_CODE = 2

    def writeCommand(self, command, sub_command=0, data='', index=0):
        self._handle.controlWrite(
            usb1.TYPE_VENDOR | usb1.RECIPIENT_DEVICE,
            self.VENDOR_COMMAND,
            (command << 8) | sub_command,
            index,
            data,
        )

    def readCommand(self, length, command, sub_command=0, index=0):
        return self._handle.controlRead(
            usb1.TYPE_VENDOR | usb1.RECIPIENT_DEVICE,
            self.VENDOR_COMMAND,
            (command << 8) | sub_command,
            index,
            length,
        )

    def _dumpList(self, offset, name_list):
        for name, value in zip(name_list, self.xpeek(offset, len(name_list))):
            if name:
                print >>sys.stderr,'%20s: %02x' % (name, ord(value))

    def dumpIRQ(self):
        self._dumpList(0xe650, (
            'EP2FIFOIE',
            'EP2FIFOIRQ',
            'EP4FIFOIE',
            'EP4FIFOIRQ',
            'EP6FIFOIE',
            'EP6FIFOIRQ',
            'EP8FIFOIE',
            'EP8FIFOIRQ',
            'IBNIE',
            'IBNIRQ',
            'NAKIE',
            'NAKIRQ',
            'USBIE',
            'USBIRQ',
            'EPIE',
            'EPIRQ',
            'GPIFIE',
            'GPIFIRQ',
            'USBERRIE',
            'USBERRIRQ',
            'ERRCNTLIM',
            'CLRERRCNT',
            'INT2IVEC',
            'INT4IVEC',
            'INTSETUP',
        ))

    def dumpUSBControl(self):
        self._dumpList(0xe680, (
            'USBCS',
            'SUSPEND',
            'WAKEUPCS',
            'TOGCTL',
            'USBFRAMEH',
            'USBFRAMEL',
            'MICROFRAME',
            'FNADDR',
        ))
        self._dumpList(0xe68a, (
            'EP0BCH',
            'EP0BCL',
        ))
        self._dumpList(0xe68d, (
            'EP1OUTBC',
        ))
        self._dumpList(0xe68f, (
            'EP1INBC',
            'EP2BCH',
            'EP2BCL',
        ))
        self._dumpList(0xe694, (
            'EP4BCH',
            'EP4BCL',
        ))
        self._dumpList(0xe698, (
            'EP6BCH',
            'EP6BCL',
        ))
        self._dumpList(0xe69c, (
            'EP8BCH',
            'EP8BCL',
        ))
        self._dumpList(0xe6a0, (
            'EP0CS',
            'EP1OUTCS',
            'EP1INCS',
            'EP2CS',
            'EP4CS',
            'EP6CS',
            'EP8CS',
            'EP2FIFOFLGS',
            'EP4FIFOFLGS',
            'EP6FIFOFLGS',
            'EP8FIFOFLGS',
            'EP2FIFOBCH',
            'EP2FIFOBCL',
            'EP4FIFOBCH',
            'EP4FIFOBCL',
            'EP6FIFOBCH',
            'EP6FIFOBCL',
            'EP8FIFOBCH',
            'EP8FIFOBCL',
            'SUDPTRH',
            'SUDPTRL',
            'SUDPTRCTL',
        ))

    def dumpUSBAV(self):
        av_name_list = [
            'SUDAV_ISR',
            'SOF_ISR',
            'SUTOK_ISR',
            'SUSPEND_ISR',
            'USBRESET_ISR',
            'HISPEED_ISR',
            'EP0ACK_ISR',
            'SPARE_ISR',
            'EP0IN _ISR',
            'EP0OUT_ISR',
            'EP1IN _ISR',
            'EP1OUT_ISR',
            'EP2_ISR',
            'EP4_ISR',
            'EP6_ISR',
            'EP8_ISR',
            'IBN_ISR',
            'SPARE_ISR',
            'EP0PING_ISR',
            'EP1PING_ISR',
            'EP2PING_ISR',
            'EP4PING_ISR',
            'EP6PING_ISR',
            'EP8PING_ISR',
            'ERRLIMIT_ISR',
            'SPARE_ISR',
            'SPARE_ISR',
            'SPARE_ISR',
            'EP2ISOERR_ISR',
            'EP2ISOERR_ISR',
            'EP2ISOERR_ISR',
            'EP2ISOERR_ISR',
        ]
        code_len = len(av_name_list) * 4
        offset = 0x3f00
        data = ''
        while len(data) < code_len:
            data += self.cpeek(offset + len(data), min(code_len - len(data), 64))
        for index, name in enumerate(av_name_list):
            ljmp, addr, pad = unpack('>BHB', data[index * 4:(index + 1) * 4])
            if name is None:
                continue
            assert ljmp == 2, ljmp
            print >>sys.stderr,'%20s=0x%04x' % (name, addr)

    def peek(self, address, length=1):
        return self.readCommand(length, self.COMMAND_MEMORY, self.COMMAND_MEMORY_INTERNAL, address)

    def xpeek(self, address, length=1):
        return self.readCommand(length, self.COMMAND_MEMORY, self.COMMAND_MEMORY_EXTERNAL, address)

    def cpeek(self, address, length=1):
        return self.readCommand(length, self.COMMAND_MEMORY, self.COMMAND_MEMORY_CODE, address)

    def poke(self, address, data):
        self.writeCommand(self.COMMAND_MEMORY, self.COMMAND_MEMORY_INTERNAL, data, address)

    def xpoke(self, address, data):
        self.writeCommand(self.COMMAND_MEMORY, self.COMMAND_MEMORY_EXTERNAL, data, address)

def USBAnalyzer(handle):
    # Free Software firmware exposes an incompatible, standard-compliant
    # configuration.
    if handle.getASCIIStringDescriptor(
                handle.getDevice()[0].getDescriptor()
            ) == 'Standard-Compliant':
        return CompliantUSBAnalyzer(handle)
    return CompatibleUSBAnalyzer(handle)

class TransferDumpCallback(object):
    __slots__ = (
        'write',
        'transfer_end_count',
        'capture_size',
        'next_measure',
        'last_measure',
        'verbose',
        'stop_condition',
        '__call__',
    )

    def __init__(self, stream, verbose=False):
        self.write = stream.write
        self.transfer_end_count = 0
        self.capture_size = 0
        self.next_measure = time.time()
        self.last_measure = (None, None)
        self.verbose = verbose
        self.stop_condition = (
            '\xf0\x41', '\xf1\x41',
            '\x41\xf0', '\x41\xf1',
        )
        self.__call__ = self.real_call

    def noop_call(self, transfer):
        return False

    def real_call(self, transfer):
        size = transfer.getActualLength()
        if not size:
            return True
        data = transfer.getBuffer()[:size]
        if data[-2:] in self.stop_condition:
            self.transfer_end_count += 1
            result = self.transfer_end_count < 2
            if not result:
                self.__call__ = self.noop_call
        else:
            result = True
            self.transfer_end_count = 0
        if self.verbose:
            self.capture_size += size
            cap_size = self.capture_size
            now = time.time()
            if now > self.next_measure:
                self.next_measure = now + 2
                last_time, last_size = self.last_measure
                self.last_measure = (now, cap_size)
                if last_size is not None:
                    sdelta = cap_size - last_size
                    if sdelta:
                        speed = sdelta / (now - last_time)
                        if speed > 1024:
                            speed /= 1024.
                            if speed > 1024:
                                speed /= 1024
                                suffix = 'M'
                            else:
                                suffix = 'k'
                        else:
                            suffix = ''
                        sys.stderr.write('\nSpeed: %.02f %sB/s\n' % (
                            speed, suffix))
                sys.stderr.write('Capture size: %i\r' % (cap_size, ))
        try:
            self.write(data)
        except IOError, exc:
            if exc.errno != errno.EPIPE:
                raise
            result = False
        return result

def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option(
        '-f', '--firmware', default='/lib/firmware/ITI1480A.rbf',
        help='Path to firmware file to upload',
    )
    parser.add_option(
        '-d', '--device',
        help='USB device to use, in "bus.dev" format',
    )
    parser.add_option(
        '-o', '--out',
        help='File to write dump data to. Default: stdout',
    )
    parser.add_option(
        '-v', '--verbose', action='store_true',
        help='Print informative messages to stderr',
    )
    (options, args) = parser.parse_args()
    if options.firmware is None:
        parser.print_help(sys.stderr)
        sys.exit(1)
    if options.device is None:
        usb_device = None
    else:
        usb_device = options.device.split('.')
        assert len(usb_device) == 2
        usb_device = (int(usb_device[0]), int(usb_device[1]))
    if options.out is None:
        out_file = os.fdopen(sys.stdout.fileno(), 'w', 0)
    else:
        out_file = open(options.out, 'wb', 0)
    verbose = options.verbose
    context = usb1.LibUSBContext()
    handle = getDeviceHandle(context, VENDOR_ID, DEVICE_ID, usb_device)
    if handle is None:
        print >>sys.stderr, 'ITI1480A USB Analyzer not found'
        sys.exit(1)
    handle.claimInterface(0)
    analyzer = USBAnalyzer(handle)
    analyzer.sendFirmware(open(options.firmware, 'rb'))

    # Call queue: process received signals synchronously.
    # Asynchronous processing is tricky because capture stop and pause need to
    # communicate with the analyzer, and complex tricks are needed when libusb
    # event handling happens "in parallel" (handleEvents + sighandler triggered
    # at the wrong time).
    call_queue = []
    def exit():
        if verbose:
            sys.stderr.write('\nExiting...\n')
        analyzer.stopCapture()
    def pause():
        analyzer.pauseCapture()
        if verbose:
            sys.stderr.write('\nCapture paused')
        os.kill(os.getpid(), signal.SIGSTOP)
        analyzer.continueCapture()
        if verbose:
            sys.stderr.write('Capture resumed\n')

    # Install signal handlers
    for sig in (signal.SIGINT, signal.SIGTERM):
        signal.signal(sig, lambda sig, stack: call_queue.append(exit))
    signal.signal(signal.SIGTSTP, lambda sig, stack: call_queue.append(pause))

    usb_file_data_reader = usb1.USBTransferHelper()
    transfer_dump_callback = TransferDumpCallback(out_file, verbose=verbose)
    usb_file_data_reader.setEventCallback(
        usb1.TRANSFER_COMPLETED,
        transfer_dump_callback,
    )

    reader_list = []
    append = reader_list.append
    for _ in xrange(64):
        data_reader = handle.getTransfer()
        data_reader.setBulk(
            0x82,
            0x8000,
            callback=usb_file_data_reader,
        )
        data_reader.submit()
        append(data_reader)

    if verbose:
        sys.stderr.write(
            'Capture started\n'
            'SIGTSTP (^Z) to pause capture (signals the pause to analyser)\n'
            'SIGCONT (fg) to unpause\n'
            'SIGINT (^C) / SIGTERM to gracefuly exit\n'
        )

    try:
        while any(x.isSubmitted() for x in reader_list):
            try:
                context.handleEvents()
            except usb1.USBErrorInterrupted:
                pass
            while call_queue:
                call_queue.pop(0)()
    finally:
        handle.releaseInterface(0)

if __name__ == '__main__':
    main()




# Copyright (C) 2010-2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
import os
from struct import unpack
from cStringIO import StringIO
from ply.yacc import yacc
from ply.lex import LexToken
import itertools
import sys
from ctypes import cast, POINTER, c_ushort
import platform

# Monkey-patch for ply.yacc defining startPush and push methods.
from . import incremental_yacc

PYPY = platform.python_implementation() == 'PyPy'
LITTLE_ENDIAN = sys.byteorder == 'little'
c_ushort_p = POINTER(c_ushort)

class ParsingDone(Exception):
    """
    Raised when capture end is found.

    More efficient than testing on each iteration, as there is one end per
    parsing session.
    """
    pass

class BaseAggregator(object):
    """
    Various aggregators/parsers in this file must implement this following API.
    """
    def push(self, *args, **kw):
        """
        Regular production. Invocation details are subclass-dependant.
        """
        raise NotImplementedError

    def stop(self):
        """
        Parsing is over, called to give a final opportunity to push pending
        data to next level.
        """
        pass

class NoopAggregator(BaseAggregator):
    """
    Simple wrapper for a callable aggregator.
    """
    def __init__(self, push):
        self.push = push

# RxCmd: see ISP1505A/ISP1505C datasheet
RXCMD_LINESTATE_MASK = 0x3
RXCMD_LINESTATE_SE0 = 0x0
RXCMD_LINESTATE_FS_J = 0x1
RXCMD_LINESTATE_FS_K = 0x2
RXCMD_LINESTATE_SE1 = 0x3
RXCMD_LIST = (
    (0x01, 'DATA0'),
    (0x02, 'DATA1'),
)

RXCMD_VBUS_MASK = 0x0c
RXCMD_VBUS_DICT = {
    0x00: 'VBUS < VB_SESS_END',
    0x04: 'VB_SESS_END <= VBUS < VB_SESS_VLD',
    0x08: 'VB_SESS_VLD <= VBUS < VA_VBUS_VLD',
    0x0c: 'VA_VBUS_VLD <= VBUS',
}

RXCMD_EVENT_MASK = 0x30
RXCMD_RX_ACTIVE = 0x10
RXCMD_HOST_DISCONNECT = 0x20
RXCMD_RX_ERROR = 0x30
RXCMD_EVENT_DICT = {
    RXCMD_RX_ACTIVE: 'RxActive',
    RXCMD_HOST_DISCONNECT: 'HostDisconnect',
    RXCMD_RX_ERROR: 'RxError RxActive',
}

# Event
EVENT_LS_DEVICE_CONNECTION = 0x0b
EVENT_FS_DEVICE_CONNECTION = 0x0f
EVENT_DEVICE_CHIRP = 0x15
EVENT_HOST_CHIRP = 0x18
EVENT_HS_IDLE = 0x24
EVENT_OTG_REQUEST = 0x62
EVENT_OTG_HNP = 0x69
EVENT_CAPTURE_PAUSED = 0xd0
EVENT_CAPTURE_RESUMED = 0xd1
EVENT_CAPTURE_STARTED = 0xe0
EVENT_CAPTURE_STOPPED_FIFO = 0xf0
EVENT_CAPTURE_STOPPED_USER = 0xf1

EVENT_DICT = {
    EVENT_LS_DEVICE_CONNECTION: 'LS device connection',
    EVENT_FS_DEVICE_CONNECTION: 'FS device connection',
    EVENT_DEVICE_CHIRP: 'Device chirp',
    EVENT_HOST_CHIRP: 'Host chirp',
    EVENT_HS_IDLE: 'HS idle',
    EVENT_OTG_REQUEST: 'OTG Session request',
    EVENT_OTG_HNP: 'OTG HNP (Host-role changed)',
    EVENT_CAPTURE_PAUSED: 'Capture paused',
    EVENT_CAPTURE_RESUMED: 'Capture resumed',
    EVENT_CAPTURE_STARTED: 'Capture started',
    EVENT_CAPTURE_STOPPED_FIFO: 'Capture stopped (fifo)',
    EVENT_CAPTURE_STOPPED_USER: 'Capture stopped (user)',
}

# File structure:
#   A serie of "packets" of variable length (1 to 5 bytes).
#   Important note: bytes are stored swapped by 2:
#     "Natural" order: 11 22 33 44
#     File order:      22 11 44 33
#   This description uses the "natural" order
# Packet structure:
#   First byte (bits):
#     7-6: payload type
#          00: No payload (used to increment time only)
#          01: Event, 1 byte of payload
#          10: Data, 1 byte of payload
#          11: RxCmd, 1 byte of payload
#     5-4: packet length in bytes, excluding payload
#     3-0: time increment
#   Following byte(s):
#     To be or-ed to timestamp once shifted (see example below)
#   Payload byte (when present)
#     Well... raw event/data/rxcmd value :) .
# Examples:
#   0x00:
#     7-6: 00 (no payload)
#     5-4: 00 (no other time increment byte)
#     3-0: 0000 (null time increment)
#   0xff 0x3f 0xff 0xff
#     7-6: 00 (no payload)
#     5-4: 11 (3 more time increment bytes)
#     3-0: 1111 (16 tics)
#     -> tic count = 0xf
#     Next byte:
#     (tic count) | (0xff << 4)
#     -> tic count = 0xfff
#     Next byte:
#     (tic count) | (0xff << 12)
#     -> tic count = 0xfffff
#     Next byte:
#     (tic count) | (0xff << 20)
#     -> tic count = 0xfffffff
#   0x09 0xd5 0x?? 0x2c:
#     7-6: 11 (RxCmd)
#     5-4: 01 (1 more time increment byte)
#     3-0: 1001 (5 ticks)
#     -> tic count = 0x5
#     Next byte:
#     (tic count) | (0x09 << 4)
#     -> tic count = 0x95
#     Next byte:
#     "VA_VBUS_VLD <= VBUS HostDisconnect" (RxCmd 0x2c)

# Type occupies the 2 most significant bits in first packet byte
TYPE_SHIFT = 6
TYPE_TIME_DELTA = 0x0 # Dataless packet, not shown in UI (used for big delays)
TYPE_EVENT = 0x1      # "Event" in UI
TYPE_DATA = 0x2       # "Data" in UI
TYPE_RXCMD = 0x3      # "RxCmd" in UI

# Packet length occupies the 2 next bits in first packet byte
LENGTH_SHIFT = 4
LENGTH_MASK = 0x3
# Number of tics contained in first packet byte
TIC_HEAD_MASK = 0xf

TIME_INITIAL_MULTIPLIER = 100.0 / 6 # 16.666...

def tic_to_time(tic):
    """
    Represent a tic count as a human-readable time with full precision, from
    minute to nanoseconds.
    """
    tic = int(tic * TIME_INITIAL_MULTIPLIER)
    tic, nano = divmod(tic, 1000)
    tic, micro = divmod(tic, 1000)
    tic, mili = divmod(tic, 1000)
    minute, sec = divmod(tic, 60)
    return '%03i:%02i.%03i\'%03i"%03in' % (minute, sec, mili, micro, nano)

def short_tic_to_time(tic):
    """
    Represent tic count as a human-readable time with limited precision to the
    most significant value pair:
    - minutes & seconds
    - seconds & milliseconds
    - milli- & microseconds
    - micro- & nanoseconds
    """
    # XXX: is the seconds & milliseconds needed ? Actually, minutes & seconds
    # should not be of much use here...
    tic = int(tic * TIME_INITIAL_MULTIPLIER)
    tic, nano = divmod(tic, 1000)
    tic, micro = divmod(tic, 1000)
    tic, mili = divmod(tic, 1000)
    if tic > 60:
        return '%i m, %i s' % divmod(tic, 60)
    if tic:
        return '%i.%03i s' % (tic, mili)
    if mili:
        return '%i ms, %i us' % (mili, micro)
    return '%i us, %i ns' % (micro, nano)

TIC_TO_MICROSECOND = TIME_INITIAL_MULTIPLIER / 1000

def tic_to_us(tic):
    """
    Convert a tic count in a floating-point microsecond value.
    """
    return tic * TIC_TO_MICROSECOND

TIC_TO_SECOND = TIME_INITIAL_MULTIPLIER / 1000000000

def tic_to_s(tic):
    """
    Convert a tic count in a floating-point second value.
    """
    return tic * TIC_TO_SECOND

RXCMD_VBUS_HL_DICT = {
    0x0: 'OTG VBus off',
    0x4: 'OTG Session end',
    0x8: 'OTG Session start',
    0xc: 'OTG VBus on',
}

# Duration of an SE0 state after which a reset is detected.
# See 7.1.7.5
MIN_RESET_TIC = 10000000 / TIME_INITIAL_MULTIPLIER # 10ms
MIN_RESET_FS_TO_CHIRP = 3000000 / TIME_INITIAL_MULTIPLIER # 3ms
MIN_LS_FS_RESET_TIC = 2500 / TIME_INITIAL_MULTIPLIER # 2.5 us
# See 7.1.13.2
MIN_LS_EOP_TIC = 670 / TIME_INITIAL_MULTIPLIER # 670 ns
MIN_FS_EOP_TIC = 82 / TIME_INITIAL_MULTIPLIER # 82 ns

# CRC5 & CRC16 computation: see 8.3.5
# Implementation detail: as python integers are not size-constrined,
# left-shifts are more complex (shift + mask) than right shifts
# (just shift). CRC remainder is right-shifted, and bit-swapped before
# returning. As a result, CRC polynomial is also byte-shifted (but once only,
# at module load).
def _swap5(value):
    return (
        ((value & 0x01) << 4) |
        ((value & 0x02) << 2) |
         (value & 0x04) |
        ((value & 0x08) >> 2) |
        ((value & 0x10) >> 4)
    )

def _swap16(value):
    value = ((value >> 1) & 0x5555) | ((value & 0x5555) << 1)
    value = ((value >> 2) & 0x3333) | ((value & 0x3333) << 2)
    value = ((value >> 4) & 0x0f0f) | ((value & 0x0f0f) << 4)
    value = ((value >> 8) & 0x00ff) | ((value & 0x00ff) << 8)
    return value

CRC5_POLYNOMIAL  = 0b00101
CRC5_RESIDUAL    = 0b01100
_CRC5_POLYNOMIAL = _swap5(CRC5_POLYNOMIAL)
CRC16_POLYNOMIAL = 0b1000000000000101
CRC16_RESIDUAL   = 0b1000000000001101
_CRC16_POLYNOMIAL = _swap16(CRC16_POLYNOMIAL)

def crc5(data):
    remainder = 0x1f
    for _, byte in data:
        for _ in xrange(8):
            xor_poly = (byte ^ remainder) & 1
            remainder >>= 1
            byte >>= 1
            if xor_poly:
                remainder ^= _CRC5_POLYNOMIAL
    return _swap5(remainder)

def crc16(data):
    remainder = 0xffff
    for _, byte in data:
        for _ in xrange(8):
            xor_poly = (byte ^ remainder) & 1
            remainder >>= 1
            byte >>= 1
            if xor_poly:
                remainder ^= _CRC16_POLYNOMIAL
    return _swap16(remainder)

# Standard USB PIDs.
PID_OUT = 0x1
PID_ACK = 0x2
PID_DATA0 = 0x3
PID_PING = 0x4
PID_SOF = 0x5
PID_NYET = 0x6
PID_DATA2 = 0x7
PID_SPLIT = 0x8
PID_IN = 0x9
PID_NAK = 0xa
PID_DATA1 = 0xb
PID_PRE = PID_ERR = 0xc
PID_SETUP = 0xd
PID_STALL = 0xe
PID_MDATA = 0xf

TOKEN_NAME = {
    PID_OUT: 'OUT',
    PID_PING: 'PING',
    PID_IN: 'IN',
    PID_SETUP: 'SETUP',
}

def _decodeToken(data):
    crc = data[2][1]
    addr = data[1][1]
    return {
        'name': TOKEN_NAME[data[0][1] & 0xf],
        'address': addr & 0x7f,
        'endpoint': (addr >> 7) | ((crc & 0x7) << 1),
        'crc': crc >> 3,
        'crc_error': crc5(data[1:]) != CRC5_RESIDUAL,
    }

DATA_NAME = {
    PID_DATA0: 'DATA0',
    PID_DATA1: 'DATA1',
    PID_DATA2: 'DATA2',
    PID_MDATA: 'MDATA',
}

def _decodeDATA(data):
    return {
        'name': DATA_NAME[data[0][1] & 0xf],
        'data': ''.join(chr(x[1]) for x in data[1:-2]),
        'crc': data[-1][1] | (data[-2][1] << 8),
        'crc_error': crc16(data[1:]) != CRC16_RESIDUAL,
    }

SPLIT_ENDPOINT_TYPE_ISOCHRONOUS = 0x01 << 1
SPLIT_ENDPOINT_TYPE_NAME = {
    0x00: 'Control',
    SPLIT_ENDPOINT_TYPE_ISOCHRONOUS: 'Isochronous',
    0x02 << 1: 'Bulk',
    0x03 << 1: 'Interrupt',
}

SPLIT_ENDPOINT_CONTINUATION = {
    (0, 0): 'middle',
    (0, 1): 'end',
    (1, 0): 'beginning',
    (1, 1): 'whole',
}

def _decodeSPLIT(data):
    endpoint_type = data[3][1] & 0x6
    result = {
        'address': data[1][1] & 0x7,
        'port': data[2][1] & 0x7,
        'endpoint_type': SPLIT_ENDPOINT_TYPE_NAME[endpoint_type],
        'crc': data[3][1] >> 3,
        'crc_error': crc5(data[1:]) != CRC5_RESIDUAL,
    }
    speed = data[2][1] >> 3
    end = data[3][1] & 0x1
    if endpoint_type == SPLIT_ENDPOINT_TYPE_ISOCHRONOUS:
        result['continuation'] = SPLIT_ENDPOINT_CONTINUATION[(speed, end)]
    else:
        result['speed'] = speed
    return result

def _decodeSSPLIT(data):
    result = _decodeSPLIT(data)
    result['name'] = 'SSPLIT'
    return result

def _decodeCSPLIT(data):
    result = _decodeSPLIT(data)
    result['name'] = 'CSPLIT'
    return result

def _decodeSOF(data):
    crc = data[2][1]
    return {
        'name': 'SOF',
        'frame': data[1][1] | ((crc & 0x7) << 8),
        'crc': crc >> 3,
        'crc_error': crc5(data[1:]) != CRC5_RESIDUAL,
    }

MESSAGE_RAW = 0
MESSAGE_RESET = 1
MESSAGE_TRANSACTION = 2
MESSAGE_TRANSFER = 3
MESSAGE_TRANSACTION_ERROR = 4
MESSAGE_TRANSFER_ERROR = 5
MESSAGE_LS_EOP = 6
MESSAGE_FS_EOP = 7
MESSAGE_INCOMPLETE = 8
MESSAGE_FS_TO_CHIRP = 9

TOKEN_TYPE_OUT = 'OUT'
TOKEN_TYPE_ACK = 'ACK'
TOKEN_TYPE_DATA0 = 'DATA0'
TOKEN_TYPE_PING = 'PING'
TOKEN_TYPE_SOF = 'SOF'
TOKEN_TYPE_NYET = 'NYET'
TOKEN_TYPE_DATA2 = 'DATA2'
TOKEN_TYPE_IN = 'IN'
TOKEN_TYPE_NAK = 'NAK'
TOKEN_TYPE_DATA1 = 'DATA1'
TOKEN_TYPE_PRE_ERR = 'PRE_ERR'
TOKEN_TYPE_SETUP = 'SETUP'
TOKEN_TYPE_STALL = 'STALL'
TOKEN_TYPE_MDATA = 'MDATA'
TOKEN_TYPE_SSPLIT = 'SSPLIT'
TOKEN_TYPE_CSPLIT = 'CSPLIT'

TRANSACTION_DECODER_DICT = {
    TOKEN_TYPE_OUT: _decodeToken,
    TOKEN_TYPE_ACK: lambda _: {'name': 'ACK'},
    TOKEN_TYPE_DATA0: _decodeDATA,
    TOKEN_TYPE_PING: _decodeToken,
    TOKEN_TYPE_SOF: _decodeSOF,
    TOKEN_TYPE_NYET: lambda _: {'name': 'NYET'},
    TOKEN_TYPE_DATA2: _decodeDATA,
    TOKEN_TYPE_SSPLIT: _decodeSSPLIT,
    TOKEN_TYPE_CSPLIT: _decodeCSPLIT,
    TOKEN_TYPE_IN: _decodeToken,
    TOKEN_TYPE_NAK: lambda _: {'name': 'NAK'},
    TOKEN_TYPE_DATA1: _decodeDATA,
    TOKEN_TYPE_PRE_ERR: lambda _: {'name': 'PRE/ERR'},
    TOKEN_TYPE_SETUP: _decodeToken,
    TOKEN_TYPE_STALL: lambda _: {'name': 'STALL'},
    TOKEN_TYPE_MDATA: _decodeDATA,
}

def decode(data):
    decoded = TRANSACTION_DECODER_DICT[data[0]](data[1])
    decoded['tic'] = data[1][0][0]
    return decoded

class _DummyLogger(object):
    """
    Quick hack to make ply.yacc more quiet, without hiding errors.
    """
    def warning(self, *args, **kw):
        pass

    debug = warning
    info = warning

    @staticmethod
    def error(message, *args, **kw):
        # Actually, raising right away is more aggressive than what ply does...
        # But much simpler too.
        assert not (args and kw), (message, args, kw)
        raise Exception(message % (args or kw))

    critical = error

# TODO: merge BaseYaccAggregator and _BaseYaccAggregator ? (watch out for more
# API collision...)
class _BaseYaccAggregator(object):
    _start = None
    _error_type = None

    def __init__(self, to_next, to_top):
        """
        to_next (BaseAggregator.push)
        to_top (callable)
            Irregular parser production (ie, errors or warnings). Invocation
            details are subclass-dependant.
        """
        self._to_next = to_next
        self._to_top = to_top
        self._parser = parser = yacc(
            module=self,
            start=self._start,
            debug=bool(os.environ.get('ITI1480A_DEBUG')),
            debugfile=self.__class__.__name__ + '_parser.out',
            errorlog=_DummyLogger(),
            write_tables=False,
        )
        self.to_yacc = parser.push
        parser.startPush()

    def stop(self):
        self.to_yacc(None)

    @staticmethod
    def _getTokenTic(token):
        raise NotImplementedError

    def p_error(self, p):
        """
        Default parser error handler. Displays the token causing the error,
        and internal parser state (relying on undocumented yacc internals, so
        might go away/break some day).
        """
        # XXX: relies on undocumented yacc internals.
        parser = self._parser
        error_tokens = [
            x.value for x in parser.symstack if isinstance(x, LexToken)
        ]
        if error_tokens:
            token_tic = self._getTokenTic(error_tokens[0])
        elif p is None:
            token_tic = None
        else:
            token_tic = self._getTokenTic(p.value)
        state = parser.statestack[-1]
        self._to_top(
            token_tic,
            self._error_type,
            (
                'Expected: %r in yacc state %r, got %r' % (
                    parser.action[state].keys(),
                    state,
                    p,
                ),
                error_tokens,
            ),
        )
        if error_tokens:
            # Restart parser and try again.
            if hasattr(parser, 'startPush'):
                parser.startPush()
            else:
                parser.restart()
            parser.errok()
            return p

    def p_empty(self, p):
        """empty :"""

class BaseYaccAggregator(BaseAggregator):
    """
    Base class for ply.yacc-based aggregators.
    Handles generic threading details.
    """
    _yacc_class = None

    def __init__(self, to_next, to_top):
        """
        to_next (BaseAggregator)
        to_top (callable)
            Irregular parser production (ie, errors or warnings). Invocation
            details are subclass-dependant.
        """
        self._to_next = to_next
        self._to_top = to_top
        self._thread = thread = self._yacc_class(to_next.push, to_top)
        self.__to_yacc = thread.to_yacc

    def _to_yacc(self, token_type, token_data):
        token = LexToken()
        token.type = token_type
        token.value = (token_type, token_data)
        token.lineno = 0 # TODO: file offset
        token.lexpos = 0
        self.__to_yacc(token)

    def stop(self):
        self._thread.stop()
        self._to_next.stop()

ENDPOINT0_TRANSFER_TYPE_DICT = {
    (TOKEN_TYPE_SETUP, 0): 'SETUP_OUT',
    (TOKEN_TYPE_SETUP, 0x80): 'SETUP_IN',
    (TOKEN_TYPE_IN, TOKEN_TYPE_ACK): 'IN_ACK',
    (TOKEN_TYPE_IN, TOKEN_TYPE_NAK): 'IN_NAK',
    (TOKEN_TYPE_IN, TOKEN_TYPE_STALL): 'IN_STALL',
    (TOKEN_TYPE_OUT, TOKEN_TYPE_ACK): 'OUT_ACK',
    (TOKEN_TYPE_OUT, TOKEN_TYPE_NAK): 'OUT_NAK',
    (TOKEN_TYPE_OUT, TOKEN_TYPE_NYET): 'OUT_NYET',
    (TOKEN_TYPE_OUT, TOKEN_TYPE_STALL): 'OUT_STALL',
    (TOKEN_TYPE_PING, TOKEN_TYPE_ACK): 'PING_ACK',
    (TOKEN_TYPE_PING, TOKEN_TYPE_NAK): 'PING_NAK',
}

class _Endpoint0TransferAggregator(_BaseYaccAggregator):
    tokens = ENDPOINT0_TRANSFER_TYPE_DICT.values()
    _start = 'transfers'
    _error_type = MESSAGE_TRANSFER_ERROR

    @staticmethod
    def _getTokenTic(token):
        return int(token[1][0][1][0][0])

    @staticmethod
    def p_transfers(p):
        """transfers : transfer
                     | transfers transfer
                     | empty
        """

    def p_transfer(self, p):
        """transfer : SETUP_OUT out_data in_data
                    | SETUP_OUT in_data
                    | SETUP_IN in_data out_handshake
        """
        data = [p[1]]
        data.extend(p[2])
        if len(p) == 4:
            data.extend(p[3])
        self._to_next(data[0][1][0][1][0][0], MESSAGE_TRANSFER, data)

    @staticmethod
    def p_out_handshake(p):
        """out_handshake : OUT_ACK
                         | OUT_NAK out_handshake
                         | PING_ACK out_handshake
                         | PING_NAK out_handshake
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            data = p[2]
            data.insert(0, p[1])
            p[0] = data

    @staticmethod
    def p_in_data(p):
        """in_data : IN_ACK
                   | IN_STALL
                   | IN_ACK in_data
                   | IN_NAK in_data
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            data = p[2]
            data.insert(0, p[1])
            p[0] = data

    @staticmethod
    def p_out_data(p):
        """out_data : OUT_ACK
                    | OUT_ACK out_data
                    | OUT_STALL
                    | OUT_NAK out_data
                    | PING_ACK OUT_ACK
                    | PING_ACK OUT_NYET
                    | PING_NAK out_data
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            data = p[2]
            data.insert(0, p[1])
            p[0] = data

class Endpoint0TransferAggregator(BaseYaccAggregator):
    """
    Aggregates SETUP transactions.
    """
    _yacc_class = _Endpoint0TransferAggregator
    def __init__(self, *args, **kw):
        super(Endpoint0TransferAggregator, self).__init__(*args, **kw)
        self._token_dispatcher = {
            TOKEN_TYPE_SETUP: (self._setup, self._slowSetup),
            TOKEN_TYPE_IN: (self._data, self._slowData),
            TOKEN_TYPE_OUT: (self._data, self._slowData),
            TOKEN_TYPE_PING: (self._ping, ), # No slow ping
        }

    def push(self, tic, transaction_type, data):
        """
        data (list of 2-tuples)
        - one of _TransactionAggregator.tokens
        - 2-tuple:
            - tic
            - token (in yacc terms) value
        """
        assert transaction_type == MESSAGE_TRANSACTION, transaction_type
        token_type = data[0][0]
        slow = data[0][0] == TOKEN_TYPE_PRE_ERR
        if slow:
            token_type = data[1][0]
        self._token_dispatcher[token_type][slow](data)

    def __setup(self, offset, data):
        self._to_yacc(
            ENDPOINT0_TRANSFER_TYPE_DICT[(
                TOKEN_TYPE_SETUP,
                data[offset][1][1][1] & 0x80,
            )],
            data,
        )

    def _setup(self, data):
        self.__setup(1, data)

    def _slowSetup(self, data):
        self.__setup(3, data)

    def _ping(self, data):
        self._to_yacc(
            ENDPOINT0_TRANSFER_TYPE_DICT[(
                TOKEN_TYPE_PING,
                data[-1][0],
            )],
            data,
        )

    def __data(self, offset, data):
        try:
            token_type = ENDPOINT0_TRANSFER_TYPE_DICT[(
                data[offset][0],
                data[-1][0],
            )]
        except KeyError:
            self._to_top(
                data[offset][1][0][0],
                MESSAGE_RAW,
                ('Unexpected ep0 transfer token', data),
            )
        else:
            self._to_yacc(token_type, data)

    def _data(self, data):
        self.__data(0, data)

    def _slowData(self, data):
        self.__data(1, data)


class PipeAggregator(BaseAggregator):
    """
    Separates USB transactions per address and endpoint.
    """
    def __init__(self, to_next, to_top, newHub, newPipe):
        """
        to_next (BaseAggregator)
            Used for address-less transactions (SOF).
        to_top
            (unused)
        newHub (callable)
            Receives device address as parameter.
        newPipe (callable)
            Receives device address and endoint as parameters.
        newHub and newPipe must return a BaseAggregator subclass instance.
        """
        self._to_top = to_top
        self._pipe_dict = {}
        self._hub_dict = {}
        self._to_next = to_next
        self._newPipe = newPipe
        self._newHub = newHub

    def _getHub(self, address):
        """
        Return hub at given address. Create one if none is known.
        """
        try:
            result = self._hub_dict[address]
        except KeyError:
            self._hub_dict[address] = result = self._newHub(address)
        return result

    def _getPipe(self, address, endpoint):
        """
        Return pipe at given address and endpoint. Create one if none is known.
        """
        try:
            device = self._pipe_dict[address]
        except KeyError:
            aggregator = self._newPipe(address, endpoint)
            self._pipe_dict[address] = {
                endpoint: aggregator,
            }
        else:
            try:
                aggregator = device[endpoint]
            except KeyError:
                device[endpoint] = aggregator = self._newPipe(address, endpoint)
        return aggregator

    def push(self, tic, transaction_type, data):
        """
        tic & transaction_type: passed through
        data: list of USB transactions
        Parses the first transaction (and possibly second, if first is a
        low-speed marker) to know destination address and endpoint, and passes
        all parameters to appropriate BaseAggregator instance's "push".
        """
        try:
            decoded = decode(data[1 if data[0][0] == TOKEN_TYPE_PRE_ERR else 0])
        except IndexError:
            return
        address = decoded.get('address')
        if address is None:
            aggregator = self._to_next
            # XXX: should it be broadcast to all device & endpoint
            # aggregators ?
        else:
            endpoint = decoded.get('endpoint')
            if endpoint is None:
                aggregator = self._getHub(address)
            else:
                aggregator = self._getPipe(address, endpoint)
        aggregator.push(tic, transaction_type, data)

    def stop(self):
        for device in self._pipe_dict.itervalues():
            for aggregator in device.itervalues():
                aggregator.stop()
        self._to_next.stop()

TRANSACTION_TYPE_DICT = {
    PID_OUT: TOKEN_TYPE_OUT,
    PID_ACK: TOKEN_TYPE_ACK,
    PID_DATA0: TOKEN_TYPE_DATA0,
    PID_PING: TOKEN_TYPE_PING,
    PID_SOF: TOKEN_TYPE_SOF,
    PID_NYET: TOKEN_TYPE_NYET,
    PID_DATA2: TOKEN_TYPE_DATA2,
    PID_IN: TOKEN_TYPE_IN,
    PID_NAK: TOKEN_TYPE_NAK,
    PID_DATA1: TOKEN_TYPE_DATA1,
    PID_PRE: TOKEN_TYPE_PRE_ERR,
    PID_SETUP: TOKEN_TYPE_SETUP,
    PID_STALL: TOKEN_TYPE_STALL,
    PID_MDATA: TOKEN_TYPE_MDATA,
}

NEED_HANDSHAKE_LIST = ('SETUP', 'IN', 'OUT')
HANDSHAKE_LIST = ('ACK', 'NAK', 'STALL', 'NYET')

class _TransactionAggregator(_BaseYaccAggregator):
    tokens = TRANSACTION_TYPE_DICT.values() + [TOKEN_TYPE_SSPLIT, TOKEN_TYPE_CSPLIT]
    _start = 'transactions'
    _error_type = MESSAGE_TRANSACTION_ERROR

    @staticmethod
    def _getTokenTic(token):
        return token[1][0][0]

    @staticmethod
    def p_transactions(p):
        """transactions : transaction
                        | transactions transaction
                        | empty
        """

    def p_transaction(self, p):
        """transaction : SETUP DATA0 ACK
                       | SSPLIT token data handshake
                       | SSPLIT token data
                       | SSPLIT token
                       | SSPLIT token handshake
                       | CSPLIT token data
                       | CSPLIT token PRE_ERR
                       | CSPLIT token
                       | CSPLIT PRE_ERR
                       | CSPLIT token handshake
                       | PRE_ERR SETUP PRE_ERR DATA0 ACK
                       | IN data ACK
                       | IN data
                       | IN NAK
                       | IN STALL
                       | PRE_ERR IN low_speed_data PRE_ERR ACK
                       | PRE_ERR IN NAK
                       | PRE_ERR IN STALL
                       | OUT data handshake
                       | OUT data
                       | PRE_ERR OUT PRE_ERR low_speed_data low_speed_handshake
                       | PING ACK
                       | PING NAK
                       | PING STALL
                       | SOF
        """
        self._to_next(p[1][1][0][0], MESSAGE_TRANSACTION, p[1:])

    def p_error(self, p):
        # XXX: relying on undocumented properties
        parser = self._parser
        error_tokens = [x.value for x in parser.symstack if isinstance(x, LexToken)]
        if error_tokens:
            self._to_next(error_tokens[0][1][0][0], MESSAGE_INCOMPLETE, error_tokens)
            # Restart parser and try again.
            if hasattr(parser, 'startPush'):
                parser.startPush()
            else:
                parser.restart()
            parser.errok()
            return p

    @staticmethod
    def p_token(p):
        """token : IN
                 | OUT
                 | SETUP
        """
        p[0] = p[1]

    @staticmethod
    def p_data(p):
        """data : low_speed_data
                | DATA2
                | MDATA
        """
        p[0] = p[1]

    @staticmethod
    def p_low_speed_data(p):
        """low_speed_data : DATA0
                          | DATA1
        """
        p[0] = p[1]

    @staticmethod
    def p_handshake(p):
        """handshake : low_speed_handshake
                     | NYET
        """
        p[0] = p[1]

    @staticmethod
    def p_low_speed_handshake(p):
        """low_speed_handshake : ACK
                               | NAK
                               | STALL
        """
        p[0] = p[1]

class TransactionAggregator(BaseYaccAggregator):
    """
    Aggregates consecutive USB packets into USB transactions.
    Validates PID value of each packet.

    to_next.push receives 3 parameters:
    - first tic in transaction
    - MESSAGE_TRANSACTION
    - list of USB packets
    to_top receives 3 parameters:
    - packet tic
    - MESSAGE_RAW
    - string
    """
    _yacc_class = _TransactionAggregator

    def push(self, packet):
        """
        packet (list of 2-tuples)
            - tic (int)
            - data (int, one byte)
        """
        assert packet
        tic, pid = packet[0]
        cannon_pid = pid & 0xf
        if cannon_pid != pid >> 4 ^ 0xf:
            self._to_top(tic, MESSAGE_RAW, '(bad pid) 0x' + ' 0x'.join('%02x' % (x[1], ) for x in packet))
            return
        try:
            trans_type = TRANSACTION_TYPE_DICT[cannon_pid]
        except KeyError:
            if cannon_pid == PID_SPLIT:
                trans_type = (packet[1][1] & 0x80) and TOKEN_TYPE_CSPLIT or TOKEN_TYPE_SSPLIT
            else:
                raise
        self._to_yacc(trans_type, packet)

class Packetiser(BaseAggregator):
    """
    Aggregates consecutive data bytes with rxActive enabled into USB packets.
    Also, handles data-less bus events (reset, device [dis]connection, bus
    voltage changes...).
    """
    _rxactive = False
    _reset_start_tic = None
    _vbus = None
    _connected = False
    _device_chirp = False
    _high_speed = False # Bus operating speed
    _high_speed_device = False # Device operating speed
    _full_speed_device = False # Device connected as FS
    _reset_start_high_speed = False

    def __init__(self, to_next, to_top, verbose=False):
        """
        to_next (BaseAggregator)
            "push" is called with a list of 2-tuples:
            - tic
            - data byte
        to_top (callable)
            Called with 3 parameters:
            - tic
            - event type (MESSAGE_RAW, MESSAGE_RESET)
            - event
        verbose (bool)
        """
        self._type_dict = {
            TYPE_EVENT: self._event,
            TYPE_DATA: self._data,
            TYPE_RXCMD: self._rxcmd,
        }
        self._to_next = to_next
        self._real_to_top = to_top
        self._data_list = []
        self._reset_queue = []
        self._verbose = verbose

    def _to_top(self, *args, **kw):
        if self._reset_start_tic is None:
            self._real_to_top(*args, **kw)
        else:
            self._reset_queue.append((args, kw))

    def push(self, tic, packet_type, data):
        """
        tic (int)
        packet_type
            One of TYPE_EVENT, TYPE_RXCMD, TYPE_DATA
        data (int)
            Byte.
        """
        # TODO: recognise low-speed keep-alive.
        if self._reset_start_tic is not None and packet_type != TYPE_EVENT and (
                    packet_type != TYPE_RXCMD or
                    data & (
                        RXCMD_EVENT_MASK | RXCMD_LINESTATE_MASK
                    ) != RXCMD_LINESTATE_SE0
                ):
            duration = tic - self._reset_start_tic
            if duration >= MIN_RESET_FS_TO_CHIRP and \
                    self._full_speed_device:
                se0_type = MESSAGE_FS_TO_CHIRP
            elif duration >= MIN_RESET_TIC:
                se0_type = MESSAGE_RESET
            elif duration >= MIN_LS_FS_RESET_TIC and \
                    not self._high_speed_device:
                se0_type = MESSAGE_RESET
            elif duration >= MIN_LS_EOP_TIC:
                se0_type = MESSAGE_LS_EOP
            elif duration >= MIN_FS_EOP_TIC:
                se0_type = MESSAGE_FS_EOP
            else:
                se0_type = None
            if se0_type is None:
                pass
            elif se0_type != MESSAGE_RESET or \
                    not self._reset_start_high_speed or \
                    not self._high_speed:
                self._real_to_top(self._reset_start_tic, se0_type, duration)
            if self._reset_queue:
                for args, kw in self._reset_queue:
                    self._real_to_top(*args, **kw)
                del self._reset_queue[:]
            self._reset_start_tic = None
        self._type_dict[packet_type](tic, data)

    def stop(self):
        # TODO: flush any pending reset ? requires knowing last tic before
        # stop was called
        if self._data_list:
            self._to_next.push(self._data_list)
            self._data_list = []
        self._to_next.stop()

    def _event(self, tic, data):
        try:
            caption = EVENT_DICT[data]
        except KeyError:
            if self._verbose:
                self._to_top(
                    tic,
                    MESSAGE_RAW,
                    '(unknown event 0x%02x)' % (data, ),
                )
        else:
            self._to_top(tic, MESSAGE_RAW, caption)
        if data == EVENT_FS_DEVICE_CONNECTION:
            self._full_speed_device = True
            self._connected = True
        elif data == EVENT_LS_DEVICE_CONNECTION:
            self._connected = True
        elif data == EVENT_DEVICE_CHIRP:
            self._device_chirp = True
        elif data == EVENT_HOST_CHIRP and self._device_chirp:
            self._high_speed_device = True
            self._high_speed = True
        elif data == EVENT_HS_IDLE:
            self._high_speed = False
        elif data in (EVENT_CAPTURE_STOPPED_FIFO, EVENT_CAPTURE_STOPPED_USER):
            raise ParsingDone

    def _data(self, tic, data):
        assert self._rxactive
        self._data_list.append((tic, data))

    def _rxcmd(self, tic, data):
        # TODO:
        # - RxError
        rxactive = data & RXCMD_RX_ACTIVE
        if self._rxactive and not rxactive and self._data_list:
            self._to_next.push(self._data_list)
            self._data_list = []
        self._rxactive = rxactive
        if data & RXCMD_HOST_DISCONNECT and self._connected:
            rendered = 'Device disconnected'
            self._connected = False
            self._full_speed_device = False
            self._device_chirp = False
            self._high_speed_device = False
            self._high_speed = False
        else:
            if self._reset_start_tic is None and \
                    data & RXCMD_LINESTATE_MASK == RXCMD_LINESTATE_SE0:
                # Maybe a reset, detect on next data
                self._reset_start_tic = tic
                self._reset_start_high_speed = self._high_speed
            vbus = data & RXCMD_VBUS_MASK
            if vbus == self._vbus:
                return
            self._vbus = vbus
            rendered = RXCMD_VBUS_HL_DICT[vbus]
        self._to_top(tic, MESSAGE_RAW, rendered)

class ReorderedStream(BaseAggregator):
    """
    Transfor a serie of data chunks in .usb file order into tic count, type
    and data values.
    Incomplete chunk tail is preserved to be decoded with next data block.
    """
    def __init__(self, out):
        """
        out (BaseAggregator)
            "push" receives 3 parameters:
            - tic count (arbitrarily long integer)
            - type (TYPE_EVENT, TYPE_DATA or TYPE_RXCMD)
            - data (1-byte integer)
        """
        self._remain = ()
        self._out = out
        self._tic = 0

    def push(self, data):
        """
        data (string)
            File chunk to process.
        """
        if len(data) % 2:
            raise ValueError('data len must be even')
        out = self._out.push
        tic = self._tic
        if LITTLE_ENDIAN and not PYPY:
            data_short_list = cast(data, c_ushort_p)
            reader = (data_short_list[x] for x in xrange(len(data) / 2))
        else:
            read = StringIO(data).read
            reader = (
                unpack('<H', read(2))[0]
                for x in xrange(0, len(data) - 1, 2)
            )
        next_data = itertools.chain(self._remain, reader).next
        try:
            while True:
                try:
                    p1 = next_data()
                except StopIteration:
                    self._remain = ()
                    break
                head = p1 >> 8
                packet_type = head >> TYPE_SHIFT
                packet_len = (head >> LENGTH_SHIFT) & LENGTH_MASK
                tic_count = head & TIC_HEAD_MASK
                if packet_len:
                    tic_count |= (p1 & 0xff) << 4
                    if packet_len > 1:
                        try:
                            p2 = next_data()
                        except StopIteration:
                            self._remain = p1,
                            break
                        tic_count |= (p2 & 0xff00) << 4
                        if packet_len > 2:
                            tic_count |= (p2 & 0xff) << 20
                            if packet_type == TYPE_TIME_DELTA:
                                tic += tic_count
                                continue
                            try:
                                p3 = next_data()
                            except StopIteration:
                                self._remain = p1, p2
                                break
                            assert p3 & 0xff == 0, hex(p3)
                            payload = p3 >> 8
                        else:
                            payload = p2 & 0xff
                    else:
                        try:
                            payload = next_data()
                        except StopIteration:
                            self._remain = p1,
                            break
                        assert payload & 0xff == 0, hex(payload)
                        payload >>= 8
                else:
                    payload = p1 & 0xff
                tic += tic_count
                out(tic, packet_type, payload)
        finally:
            self._tic = tic

    def stop(self):
        self._out.stop()




#!/usr/bin/env python
# Copyright (C) 2010-2015  Vincent Pelletier <plr.vincent@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
from setuptools import setup, find_packages

setup(
    name='ITI1480A-linux',
    description='Tools for the ITI1480A usb protocol analyser',
    keywords='usb protocol analyser ITI1480A',
    version='3.0',
    author='Vincent Pelletier',
    author_email='plr.vincent@gmail.com',
    url='http://github.com/vpelletier/ITI1480A-linux',
    license='GPLv2+',
    platforms=['any'],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'spt2hex=iti1480a.spt2hex:main',
            'iti1480a-capture=iti1480a.capture:main',
            'iti1480a-display=iti1480a.display:main',
        ],
    },
    classifiers=[
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'libusb1',
        'ply',
    ],
)

